using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;

namespace CFQuery_Reassembler {
    class QueryTextReassembler {

        public QueryTextReassembler() {
            this.OriginalText = "";
        }
        public QueryTextReassembler(String newText) {
            this.OriginalText = newText;
        }

        private String _originalText;
        private String _reassembledQuery;
        private String _reassembledCFQuery;
        private bool isReassembled;

        public String OriginalText {
            get {
                return _originalText;
            }
            set {
                _originalText = value;
                isReassembled = false;
            }
        }

        public String ReassembledQuery {
            get {
                if (!isReassembled) {
                    reassembleQuery();
                }
                return _reassembledQuery;
            }
        }

        public String ReassembledCFQuery {
            get {
                if (!isReassembled) {
                    reassembleQuery();
                }
                return _reassembledCFQuery;
            }
        }



        private void reassembleQuery() {
			/* Sample Query
			 * 
			 * 
queryName (Datasource=dopefly, Time=115ms, Records=10) in D:\Projects\TestProject\scribble.cfm @ 00:08:54.054
			
			select top 10 *
			from comments
			where Name = ?
			or Name = ?
			and subscribe = ? 
			
Query Parameter Value(s) -
Parameter #1(cf_sql_varchar) = MrNate
Parameter #2(cf_sql_varchar) = Nathan Strutz
Parameter #3(cf_sql_integer) = 0
			 * 
			 * 
			 */

			String qry = _originalText;

            // Suck out just the query for the reassembled query
            qry = qry.Trim();
            qry = Regex.Replace(qry, @"^([\w\._\$-]+\s*\(|\s*\()[^\)]*Datasource=[^\r\n]+([\s]*[\r\n]+)*", ""); // remove that first line
            //qry = Regex.Replace(qry, @"^([\r\n])?([\t ]*[\w])", "$2");  // take out any leading lines
            qry = Regex.Replace(qry, @"Query Parameter Value\(s\) -[\s\S]*$", ""); // Remove any queryparam values
            qry = qry.TrimEnd(new char[] { (char)9, (char)10, (char)11, (char)12, (char)13, (char)32 });
            
            // now we have the query alone, but it contains ? for queryparams

			// currentFindIndex helps avoid searching in replaced strings for '?' chars
			int currentFindIndex = 0;
			String matchText;
			MatchCollection qpMatches = Regex.Matches(_originalText, @"Parameter #\d+\(cf_sql_(\w+)\)\s*=([^\r\n]*)", RegexOptions.Singleline);
            foreach (Match m in qpMatches) {
                //int questionLocation = Regex.Match(qry, @"(?<!')[^']*\?[^']*(?!')").Index; // Test using lookaround to make sure we're not in a quotation block... didn't exaclty work.
                int questionLocation = qry.IndexOf("?", currentFindIndex);
                qry = qry.Remove(questionLocation, 1);
				matchText = m.Groups[2].ToString().Trim();
				if (m.Groups[2].ToString().Length>0) {
					if ("blob,char,clob,date,longvarchar,time,timestamp,varchar".IndexOf(m.Groups[1].ToString()) >= 0) {
						qry = qry.Insert(questionLocation, "'" + matchText + "'");
						currentFindIndex = questionLocation + matchText.Length + 2;
					} else {
						qry = qry.Insert(questionLocation, matchText);
						currentFindIndex = questionLocation + matchText.Length;
					}
				} else {
					qry = qry.Insert(questionLocation, "null");
					currentFindIndex = questionLocation + 4; // null = 4 chars
				}
				--currentFindIndex;

            }

            // Trim excess leading tabs (all but the last)
            String[] qryLines = qry.Split((Char)10);
            int minWhiteSpace = 1000;
            // determine lowest number
            for (int i=0; i<qryLines.Length; i++) {
                // ignore blank lines
                if (qryLines[i].Trim().Length > 0) {
					minWhiteSpace = Math.Min(minWhiteSpace, Regex.Match(qryLines[i], "(?<=\t+)[\t][^\t]").Index);
                }
                qryLines[i] += "\n";
            }

			if (minWhiteSpace != 0) {
				// trim each line the minimum amount
				for (int i = 0; i < qryLines.Length; i++) {
					// ignore blank lines
					if (qryLines[i].Trim().Length > 0) {
						qryLines[i] = qryLines[i].Remove(0, minWhiteSpace);
					}
				}
			} else {
				// Trim excess leading spaces (all but the last 4)

			}




            // rebuild sql string
            qry = String.Concat(qryLines);

            _reassembledQuery = qry;

            // Recreate the cfquery tags

            String origTxt = _originalText.Trim();
            String cfQry;
            Match match;
            cfQry = "<cfquery ";

            // Query Name
            match = Regex.Match(origTxt, @"^([\w\._-]+)");
            if (match.Success) {
                cfQry += "name=\"" + match.Groups[1].ToString() + "\" ";
            }
            // DSN
            match = Regex.Match(origTxt, @"\(Datasource=([\w_-]*),");
            cfQry += "datasource=\"" + match.Groups[1].ToString() + "\">";

            cfQry += "\r\n";
            cfQry += qry;
            cfQry += "\r\n";
            cfQry += "</cfquery>";

            _reassembledCFQuery = cfQry;

            isReassembled = true;
        }



    }
}
